;; Generated package description from cl-lib.el  -*- no-byte-compile: t -*-
(define-package "cl-lib" "0.7" "Forward cl-lib compatibility library for Emacs<24.3" 'nil :url "https://elpa.gnu.org/packages/cl-lib.html" :authors '(("Stefan Monnier" . "monnier@iro.umontreal.ca")) :maintainer '("Stefan Monnier" . "monnier@iro.umontreal.ca"))
